Public Class Link
    ' The Link's from and to nodes.
    Public Node1 As Node
    Public Node2 As Node

    ' The Link's cost.
    Public Cost As Double

    ' Pens for drawing the link.
    Private Shared m_PenNormal As Pen = Pens.Black
    Private Shared m_PenHighlighted As Pen = Pens.Red
    Private Shared m_PenSelected As Pen = Pens.Yellow

    ' Test distance from links.
    Private Shared m_LinkDistance As Double = 3

    ' Determines whether the link is drawn normally,
    ' highlighted, or selected.
    Public Enum LinkSelectStatus
        Normal
        Highlighted
        Selected
    End Enum
    Public SelectStatus As LinkSelectStatus

    ' Constructor.
    Public Sub New(ByVal new_node1 As Node, _
      ByVal new_node2 As Node, _
      Optional ByVal new_cost As Double = 1.79769313486231E+308)

        Node1 = new_node1
        Node2 = new_node2
        If new_cost > 1.79769313486231E+308 Then
            Cost = new_cost
        Else
            CostEqualsLength()
        End If
    End Sub

    ' Set the link's cost equal to its length.
    Public Sub CostEqualsLength()
        Dim dx As Single
        Dim dy As Single

        dx = Node1.X - Node2.X
        dy = Node1.Y - Node2.Y
        Cost = Math.Sqrt(dx * dx + dy * dy)
    End Sub

    ' Draw the link on the indicated canvas.
    Public Sub Draw(ByVal canvas As Graphics)

        Dim the_pen As Pen

        ' See which pen we should use.
        Select Case SelectStatus
            Case LinkSelectStatus.Normal
                the_pen = m_PenNormal
            Case LinkSelectStatus.Highlighted
                the_pen = m_PenHighlighted
            Case LinkSelectStatus.Selected
                the_pen = m_PenSelected
        End Select

        ' Draw the link.
        With canvas
            .DrawLine(the_pen, _
                Node1.X, Node1.Y, _
                Node2.X, Node2.Y)
        End With
    End Sub

    ' Return the distance from the point (px, py)
    ' to the segment (X1, Y1) --> (X2, Y2).
    Private Function DistancePointToSegment( _
      ByVal px As Integer, ByVal py As Integer, _
      ByVal X1 As Integer, ByVal Y1 As Integer, _
      ByVal X2 As Integer, ByVal Y2 As Integer) As Double

        Dim dx As Double
        Dim dy As Double
        Dim t As Double

        dx = X2 - X1
        dy = Y2 - Y1
        If dx = 0 And dy = 0 Then
            ' It's a point not a line segment.
            dx = px - X1
            dy = py - Y1
            Return Math.Sqrt(dx * dx + dy * dy)
        End If

        t = (dx * (px - X1) + dy * (py - Y1)) / (dx * dx + dy * dy)
        If t < 0 Then
            dx = px - X1
            dy = py - Y1
        ElseIf t > 1 Then
            dx = px - X2
            dy = py - Y2
        Else
            dx = px - (X1 + t * dx)
            dy = py - (Y1 + t * dy)
        End If

        Return Math.Sqrt(dx * dx + dy * dy)
    End Function

    ' Return the distance from the link to this point.
    Public Function DistanceToPoint(ByVal X As Integer, ByVal Y As Integer) As Double
        Return DistancePointToSegment(X, Y, Node1.X, Node1.Y, Node2.X, Node2.Y)
    End Function

    ' Return True if this point is on the link.
    Public Function IsAt(ByVal test_x As Integer, _
                         ByVal test_y As Integer) As Boolean
        Return (DistanceToPoint(test_x, test_y) <= m_LinkDistance)
    End Function
End Class
